cordova.define("com-anthem-mobile-plugins-biometrics.AnthemTouchID", function(require, exports, module) {
var exec = require('cordova/exec');

/**
 * Represents the JS API for Anthem Touch ID.
 * @constructor
 */
var AnthemTouchID = function() {
  this.name = 'AnthemTouchID';
};

/**
 * Prompts for touch ID authentication, showing the given message.
 * @param {Function} successCallback Called when cordova.exec succeeds.
 * @param {Function} errorCallback Called when cordova.exec fails.
 * @param {string} message Message shown to the user.
 */
AnthemTouchID.prototype.authenticate = function(successCallback, errorCallback, message) {
    exec(successCallback, errorCallback, 'AnthemTouchID', 'authenticate', [message]);
};

/**
 * Prompts for touch ID authentication, showing the given message.
 * @param {Function} successCallback Called when cordova.exec succeeds.
 * @param {Function} errorCallback Called when cordova.exec fails.
 * @param {string} message Message shown to the user.
 */
AnthemTouchID.prototype.reAuthenticate = function(successCallback, errorCallback, message) {
    exec(successCallback, errorCallback, 'AnthemTouchID', 'reAuthenticate', [message]);
};

/**
 * Checks support for touch ID on the current device.
 * @param {Function} successCallback Called when cordova.exec succeeds.
 * @param {Function} errorCallback Called when cordova.exec fails.
 */
AnthemTouchID.prototype.checkSupport = function(successCallback, errorCallback) {
    exec(successCallback, errorCallback, 'AnthemTouchID', 'checkSupport', []);
};

/**
 * Dismisses any touch ID prompt on the current device.
 * @param {Function} successCallback Called when cordova.exec succeeds.
 * @param {Function} errorCallback Called when cordova.exec fails.
 */
AnthemTouchID.prototype.dismissPrompt = function(successCallback, errorCallback) {
    exec(successCallback, errorCallback, 'AnthemTouchID', 'dismissPrompt', []);
};

module.exports = new AnthemTouchID();

});
