cordova.define("com-anthem-mobile-secure-storage.SecureStorage", function(require, exports, module) {
var exec = require('cordova/exec');

/**
 * Represents the JS API for Anthem Keychain.
 * @constructor
 */
var SecureStorage = function () {
    this.name = 'SecureStorage';
};

/**
 * Stores a value in the device keychain with the given key.
 * @param {Function} successCallback Called when cordova.exec succeeds.
 * @param {Function} errorCallback Called when cordova.exec fails.
 * @param {string} key Key in keychain.
 * @param {string} value Value in keychain.
 */
SecureStorage.prototype.saveValue = function (successCallback, errorCallback, key, value) {
    exec(successCallback, errorCallback, 'SecureStorage', 'saveValue', [key, value]);
};

/**
 * Deletes any value in the device keychain for the given key.
 * @param {Function} successCallback Called when cordova.exec succeeds.
 * @param {Function} errorCallback Called when cordova.exec fails.
 * @param {string} key Key to delete in the device keychain.
 */
SecureStorage.prototype.deleteValue = function (successCallback, errorCallback, key) {
    exec(successCallback, errorCallback, 'SecureStorage', 'deleteValue', [key]);
};

/**
 * Loads the value associated with the given key in the device keychain.
 * @param {Function} successCallback Called when cordova.exec succeeds.
 * @param {Function} errorCallback Called when cordova.exec fails.
 * @param {string} key Key to load from the device keychain.
 */
SecureStorage.prototype.loadValue = function (successCallback, errorCallback, key) {
    exec(successCallback, errorCallback, 'SecureStorage', 'loadValue', [key]);
};

/**
 * Clears any values typically saved by SecureStorage.
 * @param {Function} successCallback Called when cordova.exec succeeds.
 * @param {Function} errorCallback Called when cordova.exec fails.
 * @param {Array<String>} keys Keys to delete.
 */
SecureStorage.prototype.clearValues = function (successCallback, errorCallback, keys) {
    exec(successCallback, errorCallback, 'SecureStorage', 'clearValues', keys || []);
};

module.exports = new SecureStorage();

});
